@@ -59,6 +59,27 @@ role {
 		my $name         = $finished_job->stage->name;
 		my $stage        = $self->_stages->{$name};
 		$stage->_analyse_completion_state($finished_job);
+		if ($stage->is_fail
+				&& $stage->_has_should_choose_retry
+				&& $stage->_chosen_retries < $stage->choose_retries
+				&& $stage->should_choose_retry($finished_job->stats,$finished_job->stderr) ) {
+			$stage->_set_state('retry');
+			$stage->_chosen_retries($stage->_chosen_retries+1);
+			$self->info( "Using user chosen retry: ",
+				$stage->_chosen_retries,
+				" of ",
+				$stage->choose_retries,
+			);
+		}
+		if ($stage->is_fail && $stage->_forced_retries < $stage->force_retries) {
+			$stage->_set_state('retry');
+			$stage->_forced_retries($stage->_forced_retries+1);
+			$self->info( "Using forced retry: ",
+				$stage->_forced_retries,
+				" of ",
+				$stage->force_retries,
+			);
+		}
 		my $state = $stage->state;
 		$self->info("stage($name) completion state: $state");
 		return $stage;
@@ -762,15 +783,6 @@ other stage or the attempt is aborted).
 		my $self = shift;
 		while (1) {
 			my $stage = $self->wait_for_completed_stage;
-			if ($stage->is_fail && $stage->_forced_retries < $stage->force_retries) {
-				$stage->_set_state('retry');
-				$stage->_forced_retries($stage->_forced_retries+1);
-				$self->info( "Using forced retry: ",
-				   	$stage->_forced_retries,
-				   	" of ",
-				   	$stage->force_retries,
-				);
-			}
 			return $stage unless $stage->is_retry;
 			$self->retry($stage);
 		}
