@@ -501,6 +501,48 @@ sub is_complete {
     return $self->is_pass || $self->is_fail;
 }
 
+=head2 should_choose_retry, choose_retries
+
+If a cluster has a tendency to fail randomly, it may be desired to
+retry a stage that fails rather than simply treating it as an unrecoverable
+failure.
+
+This pair of attributes specifies two aspects of making such a choice.
+
+First, the should_choose_retry attribute can be given a code reference to code
+that returns true if a retry should be made.  The sub is called with two arguments:
+the status hash containing the known status results from the most recent try to
+run the stage, and a string with the name of the stderr output file.  This code
+returns a true value to indicate that a retry should be attempted.  If this
+attribute is not explicitly provided, it defaults to code that always returns 0
+to not cause a retry attempt.
+
+Second, the choose_retries attribute specifies number of times that the stage
+should be retried at the request of the should_choose_retry code.
+The default is 1.
+
+=cut
+
+has 'should_choose_retry' => (
+	is      => 'ro',
+	isa     => 'CodeRef',
+	predicate => '_has_should_choose_retry'
+);
+
+has 'choose_retries' => (
+	is        => 'ro',
+	isa       => 'Int',
+	lazy      => 1,
+	default   => sub { my $self = shift; $self->_has_should_choose_retry ? 1 : 0 }
+);
+
+has '_chosen_retries' => (
+	is        => 'rw',
+	isa       => 'Int',
+    init_arg  => undef,
+	default   => 0
+);
+
 =head2 force_retries
 
 If a cluster has a tendency to fail randomly, it may be desired to
@@ -511,6 +553,16 @@ This attribute specifies the number of times that the stage should be retried
 until it passes.  The default is to not retry (unless the underlying driver
 retries for a specific reason).
 
+There are three types of retry selections.  First, the devide driver can provide
+automatic retries for considitions appropriate to that cluster type.  Second,
+the retries selected by the should_choose_retry attribute can cause a retry if
+the user-provided code chooses.  Finally, the force_retries attribute can cause
+a retry.  Each of these three is independent - any of them can cause a retry
+regardless of what the others would select.  A stage only is deemed to have
+failed completely if none of the methods selects a retry.  The counters for each
+type of retry test is only adjusted if that test was the one that caused a retry
+so there can be as amny retries as the total of the number allowed by each type.
+
 =cut
 
 has 'force_retries' => (
